#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.interface import implementer
from zope.location import Location
from zope.schema.fieldproperty import FieldProperty

from onf_website import _
from onf_website.component.illustration.interfaces import IIllustrationParagraphRendererSettings
from pyams_content.component.illustration.interfaces import IIllustration
from pyams_content.component.paragraph.interfaces import IBaseParagraph
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_default_theme.component.illustration import BaseIllustrationRenderer
from pyams_default_theme.component.illustration.interfaces import ILLUSTRATION_AFTER_BODY, ILLUSTRATION_BEFORE_BODY, \
    ILLUSTRATION_BEFORE_TITLE
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, get_annotation_adapter


#
# Illustration paragraph renderers settings
#

ILLUSTRATION_RENDERER_SETTINGS_KEY = 'onf_website.illustration.renderer:default'


@implementer(IIllustrationParagraphRendererSettings)
class IllustrationParagraphRendererSettings(Persistent, Location):
    """Illustration paragraph renderer settings"""

    display_description = FieldProperty(IIllustrationParagraphRendererSettings['display_description'])
    description_position = FieldProperty(IIllustrationParagraphRendererSettings['description_position'])
    display_button = FieldProperty(IIllustrationParagraphRendererSettings['display_button'])
    button_label = FieldProperty(IIllustrationParagraphRendererSettings['button_label'])

    def __init__(self):
        super(IllustrationParagraphRendererSettings, self).__init__()
        self.button_label = {
            'en': "Read description",
            'fr': "Lire la description"
        }


@adapter_config(context=IIllustration, provides=IIllustrationParagraphRendererSettings)
def default_illustration_paragraph_renderer_settings_factory(context):
    """Illustration paragraph renderer settings factory"""
    return get_annotation_adapter(context, ILLUSTRATION_RENDERER_SETTINGS_KEY,
                                  IllustrationParagraphRendererSettings)


#
# Illustration paragraph renderers
#

@adapter_config(name='banner', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-banner.pt', layer=IPyAMSLayer)
class BannerIllustrationRenderer(BaseIllustrationRenderer):
    """Banner illustration renderer"""

    label = _("ONF: full width illustration banner")
    weight = 1

    position = ILLUSTRATION_BEFORE_TITLE

    def __new__(cls, context, request):
        parent = context.__parent__
        if not IBaseParagraph.providedBy(parent):
            return None
        return BaseIllustrationRenderer.__new__(cls)


@adapter_config(name='centered-before-body', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class CenteredBeforeTextIllustrationRenderer(BaseIllustrationRenderer):
    """Default illustration renderer"""

    label = _("ONF: centered illustration before text")
    weight = 2

    position = ILLUSTRATION_BEFORE_BODY
    settings_interface = IIllustrationParagraphRendererSettings


@adapter_config(name='left+zoom', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-left.pt', layer=IPyAMSLayer)
class LeftIllustrationWithZoomRenderer(BaseIllustrationRenderer):
    """Illustration renderer with small image and zoom"""

    label = _("ONF: small illustration on the left")
    weight = 3

    position = ILLUSTRATION_BEFORE_BODY


@adapter_config(name='right+zoom', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-right.pt', layer=IPyAMSLayer)
class RightIllustrationWithZoomRenderer(BaseIllustrationRenderer):
    """Illustration renderer with small image and zoom"""

    label = _("ONF: small illustration on the right")
    weight = 4

    position = ILLUSTRATION_BEFORE_BODY


@adapter_config(name='default', context=(IIllustration, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/illustration-default.pt', layer=IPyAMSLayer)
class CenteredIllustrationRenderer(BaseIllustrationRenderer):
    """Default illustration renderer"""

    label = _("ONF: centered illustration after text")
    weight = 5

    position = ILLUSTRATION_AFTER_BODY
    settings_interface = IIllustrationParagraphRendererSettings
